/*
 * NRF24L01.h
 *
 *  Created on: 22 lut 2017
 *      Author: Mateusz Kuc
 *      Email: kucmateusz92@gmail.com
 */
#ifndef NRF24L01_H_
#define NRF24L01_H_
#include <stdint.h>

//mnemoniki instrukcji
#define READ_REG        		0x00//odczyt komend w rejestrze
#define WRITE_REG       		0x20//zapis komend do rejestru
#define RD_RX_PAYLOAD     		0x61//dlugosc rejestru danych w RX
#define WR_TX_PAYLOAD     		0xA0//dlugosc rejestru danych w TX
#define FLUSH_TX      			0xE1
#define FLUSH_RX      			0xE2
#define REUSE_TX_PL   			0xE3//powtorzenie ostatniego pakietu
#define R_RX_PL_WID   			0x60//odczyt dlugosci danych
#define W_ACK_PAYLOAD 			0xA8//specyfikacja rur (pipe)
#define W_TX_PAYLOAD_NOACK		0xB0//wyslanie dlugosci danych z statusem braku potwierdzenia odbioru
#define NOP           			0xFF

//rejestr STATUSu
#define STATUS_RBANK 	0x80
#define STATUS_RX_DR 	0x40
#define STATUS_TX_DS 	0x20
#define STATUS_MAX_RT 	0x10
#define STATUS_TX_FULL 	0x01

//mapa pamieci i mnemoniki
#define	CONFIG	    0x00
#define EN_AA       0x01
#define EN_RX_ADDR  0x02
#define SETUP_AW    0x03
#define SETUP_RETR  0x04
#define RF_CH       0x05
#define RF_SETUP    0x06
#define STATUS      0x07
#define OBSERVE_TX  0x08
#define CD          0x09
#define RX_ADDR_P0  0x0A
#define RX_ADDR_P1  0x0B
#define RX_ADDR_P2  0x0C
#define RX_ADDR_P3  0x0D
#define RX_ADDR_P4  0x0E
#define RX_ADDR_P5  0x0F
#define TX_ADDR     0x10
#define RX_PW_P0    0x11
#define RX_PW_P1    0x12
#define RX_PW_P2    0x13
#define RX_PW_P3    0x14
#define RX_PW_P4    0x15
#define RX_PW_P5    0x16
#define FIFO_STATUS 0x17
#define DYNPD       0x1C
#define FEATURE		0x1D

//status dla FIFO
#define FIFO_STATUS_TX_REUSE 	0x40
#define FIFO_STATUS_TX_FULL 	0x20
#define FIFO_STATUS_TX_EMPTY 	0x10
#define FIFO_STATUS_RX_FULL 	0x02
#define FIFO_STATUS_RX_EMPTY 	0x01

//odblokowanie auto-potwierdzenia odbioru
#define ENAA_P5     5
#define ENAA_P4     4
#define ENAA_P3     3
#define ENAA_P2     2
#define ENAA_P1     1
#define ENAA_P0     0

//rejestr konfiguracyjny
#define MASK_RX_DR  6
#define MASK_TX_DS  5
#define MASK_MAX_RT 4
#define EN_CRC      3
#define CRC0        2
#define PWR_UP      1
#define PRIM_RX     0

//odblokowanie adresowania RX
#define ERX_P5      5
#define ERX_P4      4
#define ERX_P3      3
#define ERX_P2      2
#define ERX_P1      1
#define ERX_P0      0

//odblokowanie dlugosci adresowania
#define AW          0

//ustawienie automatycznej retransmisji
#define ARD         4
#define ARC         0

//stan FIFO
#define TX_REUSE    6
#define FIFO_FULL   5
#define TX_EMPTY    4
#define RX_FULL     1
#define RX_EMPTY    0

//dynamiczna dlugosc
#define DPL_P0      0
#define DPL_P1      1
#define DPL_P2      2
#define DPL_P3      3
#define DPL_P4      4
#define DPL_P5      5

//rejestr wlasciwosci
#define EN_DPL		2
#define EN_ACK_PAY	1
#define EN_DYN_ACK	0

//ustawienie rejestru RF
#define RF_DR_LOW   5
#define PLL_LOCK    4
#define RF_DR       3
#define RF_PWR      1

//rejestr do obserwacji transmitowania
#define PLOS_CNT    4
#define ARC_CNT     0



//glowny rejestr statusu
#define RX_DR       6
#define TX_DS       5
#define MAX_RT      4
#define RX_P_NO     1
#define TX_FULL     0

//definicje (nie)potwierdzenia odbioru
#define NOACK           		0x00
#define ACK           			0x01

#define CE_l    HAL_GPIO_WritePin(SPI1_CE_GPIO_Port, SPI1_CE_Pin, RESET)
#define CS_l    HAL_GPIO_WritePin(SPI1_CSN_GPIO_Port, SPI1_CSN_Pin, RESET)
#define CE_h    HAL_GPIO_WritePin(SPI1_CE_GPIO_Port, SPI1_CE_Pin, SET)
#define CS_h    HAL_GPIO_WritePin(SPI1_CSN_GPIO_Port, SPI1_CSN_Pin, SET)

void NRF24_init(void);
uint8_t SPI_reg(uint8_t reg, uint8_t val);
uint8_t SPI_r_w(uint8_t x);
void Set_Tx_Adr(uint8_t* adr);
void SPI_zapis_buf(uint8_t reg, uint8_t *buf, uint8_t dlugosc);
void Set_Rx_Adr(uint8_t rura, uint8_t * adr);
void Set_Px_Adr(uint8_t rura, uint8_t adr);
void wyslij_dane(uint8_t* dane, uint8_t ilosc, uint8_t ack);
void tryb_TX(void);
uint8_t daj_status(void);
uint8_t jest_wysylane(void);
void tryb_RX(void);
uint8_t dane_gotowe(void);
uint8_t RX_FIFO_pusty(void);
void odbierz_dane(uint8_t* data);
void SPI_odczyt_buf(uint8_t reg, uint8_t *buf, uint8_t dlugosc);
void Flush_RX(void);

#endif /* NRF24L01_H_ */

