/*
 * NRF24L01.c
 *
 *  Created on: 22 lut 2017
 *      Author: Mateusz Kuc
 *      Email: kucmateusz92@gmail.com
 */


#include "stm32f0xx_hal.h"
#include "spi.h"
#include "gpio.h"
#include "NRF24L01.h"

const uint16_t NRF24_reg[] =//ustawianie rejestrow
{
	//rejestr CONFG: bez przerwan od RX, TX, MAX_RT, wlaczone CRC, 2bitowe kodowanie CRC, POWER DOWN, RX/TX kontrolowane PTX
	(CONFIG<<8) 		| 	(1<<MASK_RX_DR) | (1<<MASK_TX_DS) | (1<MASK_MAX_RT) | (1<<EN_CRC) | (1<<CRC0) | (0<<PWR_UP) | (0<<PRIM_RX),
	//rejestr EN_AA: odblokowanie potwierdzenia danych dla każdego z 5 rur transmisyjnych
	(EN_AA<<8)  		| 	0x3F,
	//rejestr EN_RX_ADDR: odblokowanie włączenia adresowania RX dla wszystkich 5 rur (pipes)
	(EN_RX_ADDR<<8) 	| 	0x3F,
	//rejestr SETUP_AW: 5-bitowe adresowanie, wspolne dla wszystkich rur
	(SETUP_AW<<8)		| 	0x03,
	//rejestr SETUP_RETR: odpowiada za automatyczna retransmisje;
	//ARC=0x0F - 15razy bedzie powtarzania retransmisja; ARD=0x03 - czekam 1ms miedzy kolejnymi retransmisjami
	(SETUP_RETR<<8)		|	(0x03<<ARD)|(0x0F<<ARC),
	//rejestr RF_CH: ustaw czestotliwosc dla kanalu RF
	//(RF_CH<<8)			|	2,
	(RF_CH<<8)			|	0x60,
	//rejestr RF_SETU: PLL wylaczone(RF_DR_LOW);
	//predkosc przesylu w powietrzu 250kbps(RF_DR na 0 lecz RF_DR_LOW na 1 co daje 250kbps);
	//moc wyjsciowa 0dBm (RF_PWR)
	(RF_SETUP<<8)		|	(1<<RF_DR_LOW)|(0<<RF_DR)|(0x03<<RF_PWR),
	//rejestr STATUS: czyszczenie rejestru
	(STATUS<<8)			|	0xFF,
	//rejestr OBSERVE_TX: tylko do odczytu, informuje o utraconych pakietach, licznik ilosci retransmisji
	(OBSERVE_TX<<8)		|	0x00,
	//rejestr CD: tylko do odczytu, detekcja nosnej sygnalu
	(CD<<8)				|	0x00,
	//dane adresowe dla rur (od 0 do 5) od RX; liczba bajtow musi byc wpisana tak jak jest ustawiona w SETUP_AW
	(RX_ADDR_P0<<8)		|	0xE7,
	(RX_ADDR_P1<<8)		|	0xC2,
	(RX_ADDR_P2<<8)		|	0xC3,
	(RX_ADDR_P3<<8)		|	0xC4,
	(RX_ADDR_P4<<8)		|	0xC5,
	(RX_ADDR_P5<<8)		|	0xC6,
	//rejestr TX_ADDR: adres transmisji, uzywane gdy korzysta sie z PTX
	(TX_ADDR<<8)		|	0xE7,
	//rejestr TX_PW_Px (x=0..5): liczba bajtów w rurze x dla  RX, jezeli 0 to rura nie uzywana
	(RX_PW_P0<<8)		|	0x20,
	(RX_PW_P1<<8)		|	0x20,
	(RX_PW_P2<<8)		|	0x20,
	(RX_PW_P3<<8)		|	0x20,
	(RX_PW_P4<<8)		|	0x20,
	(RX_PW_P5<<8)		|	0x20,
	//rejestr FIFO: rejestr statusu FIFO, tylko do odczytu
	(FIFO_STATUS<<8)	|	0x00,
	//ustawienie dynamicznej dlugosci bloku danych dla rur, wymaga odblokowania EN_AA dla kazdej rury i EN_DPL
	(DYNPD<<8)			|	0x3F,
	//rejestr FEATURE: rejestr funkcji; EN_DPL - odblokowanie dynamicznej dlugosci bloku danych;
	//EN_ACK_PAY - odblokowanie bloku danych z potwierdzeniem (ACK);
	//EN_DYN_ACK - odblokowuje komende W_TX_PAYLOAD_NOACK
	(FEATURE<<8)		|	(1<<EN_DPL) | (1<<EN_ACK_PAY) | (1<<EN_DYN_ACK)	//
};

//funkcja SPI_r_w
uint8_t SPI_r_w(uint8_t x)
{//wysylanie przez sprzetowego SPI danych
	uint8_t z=0;
	HAL_SPI_TransmitReceive(&hspi1, &x, &z, 1, 10);
	return z;
}

//funkcja zapisu przez SPI
uint8_t SPI_reg(uint8_t reg, uint8_t val)
{
	CS_l;      				// CSN na poziom niski, inicjalizacja komunikacji po SPI z NRF
	SPI_r_w(reg);            // Select register to read from..
	val = SPI_r_w(val);    	// ..then read register value
	CS_h;         			// CSN high, terminate SPI communication
	return(val);        	// return register value
}

//Function: SPI_Read_Buf();
void SPI_odczyt_buf(uint8_t reg, uint8_t *buf, uint8_t dlugosc)
{
	uint8_t bajt;
	CS_l;//CS w stan niski
	SPI_r_w(reg);//wybor rejestru do zapisu
	for(bajt=0; bajt < dlugosc; bajt++)
		buf[bajt] = SPI_r_w(0);//odczyt po SPI
	CS_h;//CS w stan wysoki
}

//funkcja bedaca buforem dla zapisu po SPI
void SPI_zapis_buf(uint8_t reg, uint8_t *buf, uint8_t dlugosc)
{
	uint8_t bajt;
	CS_l;//ustawia Cs na stan niski i inicjalizuje komunikacje po SPI
	SPI_r_w(reg);//wybiera odpowiedni rejestr i do niego zapisuje/odczytuje
	for(bajt=0; bajt < dlugosc; bajt++)//zapisuje do buforu
		SPI_r_w(buf[bajt]);
	CS_h;//ustawia CS w stan wysoki
}

//funkcja zwracajaca status
uint8_t daj_status()
{
    uint8_t rv;

    CS_l;
    rv = SPI_r_w(NOP);
    CS_h;

    return rv;
}

/* Set channel number */
void ustaw_numer_kanalu(uint8_t kanal)
{
	SPI_reg((WRITE_REG|RF_CH), kanal);
}

//ustawianie adresu RX dla rur P2-P5
void Set_Px_Adr(uint8_t rura, uint8_t adr)
{
	SPI_reg(WRITE_REG|rura, adr);
}

//funkcja ustawiajaca adres RX
void Set_Rx_Adr(uint8_t rura, uint8_t * adr)
{
	SPI_zapis_buf(WRITE_REG|rura, adr, 5);//5 - dlugosc adresu dla NRF24
}

//funkcja ustawiajaca adres TX
void Set_Tx_Adr(uint8_t* adr)
{
	//RX_ADDR_P0 musi byc ustawiony wysylajac addr w trybie automatycznego potwierdzenia dzialania
	SPI_zapis_buf(WRITE_REG|RX_ADDR_P0, adr, 5);
    SPI_zapis_buf(WRITE_REG|TX_ADDR, adr, 5);
}

/* Checks if receive FIFO is empty or not */
uint8_t RX_FIFO_pusty()
{
    uint8_t fifo;
    fifo = SPI_reg(FIFO_STATUS, 0);
    fifo = fifo & FIFO_STATUS_RX_EMPTY;
    return fifo;
}

//zwraca ile danych czeka do odebrania w FIFO
uint8_t dlugosc_danych()
{
    uint8_t status;
    status = SPI_reg(R_RX_PL_WID, 0);
    return status;
}

//funkcja zwracajaca informacje ile razy probowano przeslac dane
uint8_t licznik_retransmisji()
{
    uint8_t ret;
    ret = SPI_reg(OBSERVE_TX, 0);
    return ret;
}

//Is sending?
uint8_t jest_wysylane()
{
    uint8_t status;
    status = daj_status();//odczytuje obecny status
    if(status&(STATUS_TX_DS|STATUS_MAX_RT))//jezeli zakonczono wysylanie badx po 15 probach nadal sie nie udalo
    {
        CE_l;//ustawiam CE w stan niski
		return 0;//jezeli skonczylo to zwraca 0
    }
    return 1;//jezeli nie skonczylo to zwraca 1
}

//funkcja sprawdzajaca czy sa jakies dane do odczytu
uint8_t dane_gotowe()
{
    uint8_t status;
    status = daj_status();
    //sprawdzam RX_DR i FIFO, samo RX_DR nie wystarczy bo moga byc jeszcze jakies dane oczekujace
    if (status & STATUS_RX_DR)//jezeli cos jest to zwracam 1 i odczytuje
    {
        return 1;
    }
	status = !RX_FIFO_pusty();
    return status;
}

//Flush Rx FIFO
void Flush_RX()
{
	SPI_reg(WRITE_REG | FLUSH_RX, 0);		//flush Rx
}

//funkcja trybu RX
void tryb_RX()
{
	uint8_t val;
	SPI_reg(WRITE_REG|FLUSH_RX, 0);//zapis rejestrow NRF
	CE_l;//ustawiam CE w stan niski
	val = daj_status();//odczytuje wartosc rejestru statusu z NRF
	val = ((val|STATUS_RX_DR)&(~(STATUS_TX_DS | STATUS_MAX_RT)));
	SPI_reg(WRITE_REG|STATUS, val);//ustawiam rejestr statusu
	val = SPI_reg(CONFIG, 0);//odczytuje wartosc rejestru CONFIG
	//ustawiam PWR_UP (zasilanie), odblokowanie CRC i RX_DR
	val = val|(1<<PRIM_RX)|(1<<PWR_UP);
	SPI_reg(WRITE_REG | CONFIG, val);
	CE_h;//ustawiam CE w stan wysoki
}

//funkcja trybu TX
void tryb_TX()
{
	uint8_t val;
	SPI_reg(WRITE_REG|FLUSH_TX, 0);//zapis rejestrow NRF
	CE_l;//ustawiam CE w stan niski
	val = daj_status();//odczytuje wartosc rejestru statusu z NRF
	val = ((val|STATUS_TX_DS|STATUS_MAX_RT)&(~STATUS_RX_DR));
	SPI_reg(WRITE_REG|STATUS, val);//ustawiam rejestr statusu
	val = SPI_reg(CONFIG, 0);//odczytuje wartosc rejestru CONFIG
	//ustawienie bitu PWR_UP (zasilania), odblokowania CRC i RX_DR
	val = ((val&(~(1<<PRIM_RX)))|(1<<PWR_UP));
	SPI_reg(WRITE_REG|CONFIG, val);
	CE_h;//ustawiam CE w stan wysoki
}

//funkcja
void Power_off()
{
	uint8_t val;
	CE_l;//CE ustawiam w stan niski
	SPI_reg(WRITE_REG|FLUSH_RX, 0);//wyrownanie RX? flush?
	SPI_reg(WRITE_REG|FLUSH_TX, 0);//wyrownanie TX?
	val = SPI_reg(CONFIG, 0);//odczyt wartosci rejestru CONFIG
	val = val & ~(1<<PWR_UP);
	SPI_reg(WRITE_REG|CONFIG, val);
}

//odczytuje dane i przechowuje w tablicy
void odbierz_dane(uint8_t* data)
{
    uint8_t dlugosc;

	dlugosc = SPI_reg(R_RX_PL_WID, 0);//odczytanie dlugosci liczby danych
	SPI_odczyt_buf(RD_RX_PAYLOAD, data, dlugosc);//odczyt danych
	SPI_reg(WRITE_REG|STATUS, STATUS_RX_DR);//zresetowanie rejestru statusu
	Flush_RX();
}

void wyslij_dane(uint8_t* dane, uint8_t ilosc, uint8_t ack)
{
    CE_l;//ustawia na Ce stan niski, wysylanie danych po SPI
    //ustaw NRF w tryb nadajnika i zalacz zasilanie w razie potrzeby
    if(ack) ack = WR_TX_PAYLOAD;//jezeli mamy ACK (makro) to z potwierdzeniem odbioru
    else ack = W_TX_PAYLOAD_NOACK;//jezeli mamy NOACK lub inna wartosc niz 1 nie jest z potwierdzeniem odbioru
    tryb_TX();//ustawienie TX
	SPI_zapis_buf(ack, dane, ilosc);//zapis dajnych do bufora SPI
    CE_h;//ustawian CE w stan wysoki i rozpoczynam transmisje
}

//Function: nRF24_Init();
void NRF24_init()
{
	uint8_t i, a;
 	uint16_t temp;
	//ile rejestrow ma ustawic
	a=(sizeof(NRF24_reg)/2);
	//inicjalizacja rejestrow NRF
	for(i=0;i<a;i++)
	{
		temp = NRF24_reg[i];
		SPI_reg((WRITE_REG|(temp>>8)), (temp&0xFF));
	}
}
